/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: ioport_icpci.c,v 1.1.2.3 1999/08/27 15:41:04 matthias Exp $
 *
 */

/*
 * Defines:
 */

#define	DEFAULT_DEVICE		"/dev/ic0ioP"

#define SAMPLING_MIN		2000 / HZ	/* intervals smaller than 20ms
						   are not useful (sched) */
#define SAMPLING_MAX		ICP_WAIT_IOP * 10	/* this is the build-in
							   timeout for read */
#define DEFAULT_SAMPLING	1000	/* 1s (in ms) is a good value */

/*
 * Includes:
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <getopt.h>
#include <time.h>

#include <sys/param.h>	/* HZ */
#include <sys/ioctl.h>

#ifdef linux
#  include <linux/limits.h>	/* PATH_MAX */
#endif

#include "icpciExt.h"

/*
 * Globals:
 */

  int dev_fd;

/*
 * Prototypes:
 */

void close_and_clear (void);

void print_stats (int);

void print_stats (int signum)
{
  struct icp_acc_t accounting_info;
  
  if (ioctl (dev_fd, GIOC_GE_GET_STATS, &accounting_info) >= 0)
    {
      printf ("statistics:\n");
      printf ("events captured %ld,\n", accounting_info.events_captured);
      printf ("bytes valid %ld,\n", accounting_info.bytes_valid);
      printf ("bytes lost %ld,\n", accounting_info.bytes_lost);
    }
  else
      printf ("no statistics available!\n");
  exit (EXIT_SUCCESS);
}
  

void
close_and_clear (void)
{

  if (close (dev_fd) < 0)
    perror ("close");
}

int
main (int argc, char *argv[])
{
  char dev_file[PATH_MAX] = DEFAULT_DEVICE;  

  int option;

  struct timeval sampling_time;

  int sampling_ms = DEFAULT_SAMPLING;

  sampling_time.tv_sec = 0;
  sampling_time.tv_usec = 0;

  while (1)
    {
      if ((option = getopt (argc, argv, "f:t:")) == -1) break;
      switch (option)
	{
	case 'f':
	  if (!optarg)
	    printf ("No or wrong special device file given, "
		    "using %s.\n", DEFAULT_DEVICE);
	  else
	    strcpy (dev_file, optarg);
	  break;
	case 't':
	  if (!optarg)
	    printf ("No or wrong sampling interval given, "
		    "using default (%dms).\n", DEFAULT_SAMPLING);
	  else
	    {
	      sampling_ms = atoi (optarg);
	      if (sampling_ms < SAMPLING_MIN || sampling_ms > SAMPLING_MAX)
		{
		  printf ("No or wrong sampling interval given, "
			  "using default (%dms).\n", DEFAULT_SAMPLING);
		  break;
		}
	    }
	  break;
	case '?':
	default:
	  printf ("Usage: ioport_icpci -f special device file "
		  "(/dev/ic0ioP) [/dev/ic[0-4]ioP]\n");
	  printf ("                    -t sampling_time (%dms) [%d-%dms]\n",
		  DEFAULT_SAMPLING, SAMPLING_MIN, SAMPLING_MAX);
	  exit (EXIT_FAILURE);
	}
    }
  
  /* use the raw device (do not wait for da data strobe) */
  if ((dev_fd = open (dev_file, /* O_RDONLY */ O_RDWR | O_NONBLOCK)) < 0)
    {
      perror ("open");
      exit (EXIT_FAILURE);
    }
  
  if (atexit (close_and_clear) < 0)
    {
      perror ("atexit");
      exit (EXIT_FAILURE);
    }

  if (signal (SIGINT, print_stats) < 0)
    {
      perror ("signal");
      exit (EXIT_FAILURE);
    }

  sampling_ms -= 1000 / HZ; /* scheduling interval */
  sampling_time.tv_sec = sampling_ms / 1000;
  sampling_time.tv_usec = (sampling_ms % 1000) * 1000;
  
  /* you can stop this only with Ctrl+C */
  for (;;)
    {
      u_char sampling_value;

      struct timeval sampling_timestamp;
      struct timezone dummy_timezone;

      if (gettimeofday (&sampling_timestamp, &dummy_timezone) < 0)
	{
	  perror ("gettimeofday");
	  exit (EXIT_FAILURE);
	}
      
      if ((read (dev_fd, &sampling_value, 1) != 1))
	{
	  perror ("read");
	  exit (EXIT_FAILURE);
	}

#if (1)	/* Don't forget to set O_RDWR in open */
      if ((write (dev_fd, &sampling_value, 1) != 1))
	{
	  perror ("write");
	  exit (EXIT_FAILURE);
	}
#endif

#ifdef __GLIBC__ /* workaround for time_t is long or not */
      printf ("%s %3ldms: 0x%0X\n",
	      strtok (ctime (&sampling_timestamp.tv_sec), "\n"),
	      sampling_timestamp.tv_usec / 1000, sampling_value);
#else
      printf ("%s %3dms: 0x%0X\n",
	      strtok (ctime ((time_t*)&sampling_timestamp.tv_sec), "\n"),
	      sampling_timestamp.tv_usec / 1000, sampling_value);
#endif
      usleep (sampling_time.tv_sec * 1000000 + sampling_time.tv_usec);
    }

  exit (EXIT_SUCCESS);
}

